<?php
	/*
	 *	Copyright 2007 John Oren
	 *	
	 *	Licensed under the Apache License, Version 2.0 (the "License");
	 *	you may not use this file except in compliance with the License.
	 *	You may obtain a copy of the License at
	 *		http://www.apache.org/licenses/LICENSE-2.0
	 *	Unless required by applicable law or agreed to in writing, software
	 *	distributed under the License is distributed on an "AS IS" BASIS,
	 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	 *	See the License for the specific language governing permissions and
	 *	limitations under the License.
	 */
	
	
	//include all of the standard fields
	require_once($path."OpenSiteAdmin/scripts/classes/Fields/Checkbox.php");
	require_once($path."OpenSiteAdmin/scripts/classes/Fields/ForeignKey.php");
	require_once($path."OpenSiteAdmin/scripts/classes/Fields/Group.php");
	require_once($path."OpenSiteAdmin/scripts/classes/Fields/Hidden.php");
	require_once($path."OpenSiteAdmin/scripts/classes/Fields/Image.php");
	require_once($path."OpenSiteAdmin/scripts/classes/Fields/Key.php");
	require_once($path."OpenSiteAdmin/scripts/classes/Fields/Label.php");
	require_once($path."OpenSiteAdmin/scripts/classes/Fields/Password.php");
	require_once($path."OpenSiteAdmin/scripts/classes/Fields/Select.php");
	require_once($path."OpenSiteAdmin/scripts/classes/Fields/Text.php");
	require_once($path."OpenSiteAdmin/scripts/classes/Fields/TextArea.php");
	
	/**
	 * Handles form field display and pre-processing.
	 * 
	 * Sets up a form field for display, handling field and error display.
	 * Performs pre-processing on a form field, preparing it for use in a database.
	 * 
	 * @author John Oren
	 * @version 1.0 December 30, 2007
	 */
	abstract class FieldManager {
		/** @var The title to display for this form field. */
		protected $title;
		/** @var The prefix to the name of this field. */
		protected $namePrefix;
		/** @var The name of the database field this form field corresponds to. */
		protected $name;
		/** @var The options and flags associated with this form field. */
		protected $options;
		/** @var MIXED Default value for this form (value currently in the database). */
		protected $default;
		/** @var True if this field should be used in a list. */
		protected $inList;
		/** @var True if this field must be filled in. */
		protected $required;
		/** @var String to hold errors messages that may need to be displayed. */
		protected $error;
		
		/**
		 * Constructs a form field with information on how to display it.
		 * 
		 * @param STRING $name The name of the database field this form field corresponds to.
		 * @param STRING $title The title to display for this form field.
		 * @param MIXED $options The options associated with this form field.
		 * @param BOOLEAN $inList True if this field should be used in a list view.
		 * @param BOOLEAN $required True if this form field is required.
		 */
		function __construct($name, $title, $options, $inList, $required=false) {
			$this->name = $name;
			if($title != null) {
				$this->title = $title.":";
			} else {
				$this->title = $title;
			}
			$this->options = $options;
			$this->inList = $inList;
			$this->required = $required;
			$this->error = null;
		}
		
		/**
		 * Prepares a form field for display.
		 * 
		 * @param MIXED $defaultValue Default value or values to display in this form field
		 * @param BOOLEAN $enabled True if this form field is editable
		 * @return STRING HTML to display for the form field
		 */
		abstract function show($defaultValue, $enabled=true);
		
		/**
		 * Prepares a form field for display in a list.
		 * 
		 * @param MIXED $defaultValue Default value or values to display for this field in a list
		 * @return STRING HTML to display in a list view
		 */
		function showListView($defaultValue) {
			return $defaultValue;
		}
		
		/**
		 * Processes the field and returns formated data ready for insertion into a SQL database.
		 * 
		 * @param MIXED $value Value or values to process
		 * @param INTEGER $mode The type of this form (one of the form type constants defined in the Form class)
		 * @return STRING Formatted string ready for insertion into a SQL database
		 */
		abstract function process($value, $mode);
		
		/**
		 * Removes formatting added by SQLPrep.
		 * 
		 * @param STRING String to be formatted
		 * @return STRING Formatted string with formatting from SQLPrep removed
		 */
		static final function formPrep($string) {
			return htmlspecialchars_decode(stripslashes($string));
		}
		
		/**
		 * Formats a string for entry into a SQL database.
		 * 
		 * @param STRING String to be formatted
		 * @return MIXED If succesful, a formatted string ready for insertion into a SQL database
		 *               If errors, an array of error messages
		 */
		static final function SQLPrep($string) {
			return htmlspecialchars(addslashes($string));
		}
		
		/**
		 * Prefixes the field with it's unique row identifier.
		 *
		 * @param INTEGER The prefix to apply to this field name.
		 */
		function setNamePrefix($prefix) {
			$this->namePrefix = $prefix;
		}
		
		/**
		 * Returns the ID of the row this field is associated with prepended to 
		 * the name of the SQL field this form field is associated with.
		 *
		 * @return STRING Row ID and name of this form field's corresponding SQL field.
		 */
		function getName() {
			return $this->namePrefix.Form::DELIMITER.$this->name;
		}
		
		/**
		 * Returns the name of the SQL field this form field is associated with.
		 *
		 * @return STRING The name of this form field's corresponding SQL field.
		 */
		function getFieldName() {
			return $this->name;
		}
		
		/**
		 * Returns the display name (title) of this form field.
		 *
		 * @return STRING The name to display with this form field.
		 */
		function getTitle() {
			return $this->title;
		}
		
		/**
		 * Returns the options and flags for this form field.
		 *
		 * @return MIXED Options for this form field.
		 */
		protected function getOptions() {
			return $this->options;
		}
		
		/**
		 * Returns the default value for this field (the value currently in the database)
		 * to be used if no value is provided
		 * 
		 * @return MIXED Default value for this field
		 */
		protected function getDefaultValue() {
			return $this->default;
		}
		
		/**
		 * Sets the default value for this field (the value currently in the database)
		 * to be used if no value is provided
		 * 
		 * @param MIXED Default value for this field
		 * @return VOID
		 */
		function setDefaultValue($default) {
			$this->default = $default;
		}
		
		/**
		 * Returns whether or not this form field should be used in a list context.
		 *
		 * @return BOOLEAN True if this field should be displayed in a list view.
		 */
		function isInList() {
			return $this->inList;
		}
		
		/**
		 * Returns whether or not this form field must be filled in.
		 *
		 * @return BOOLEAN True if this field cannot be empty.
		 */
		function isRequired() {
			return $this->required;
		}
		
		/**
		 * Sets an error message if this message is required and was not filled in
		 *
		 * @return VOID
		 */
		function setEmpty() {
			if($this->isRequired() && empty($this->error)) {
				$this->error = "This field must have a value";
			}
		}
		
		/**
		 * Returns any error messages associated with this field
		 *
		 * @return STRING Error message(s)
		 */
		function getError() {
			return $this->error;
		}
		
		/**
		 * Returns whether or not this field encountered an error
		 *
		 * @return BOOLEAN True if an error was encountered
		 */
		function hasError() {
			return $this->getError() !== null;
		}
		
		/**
		 * Returns a formated string of all error messages associated with this field
		 * Returns an empty string if no errors were encountered
		 *
		 * @return STRING Formatted error message(s) or an empty string
		 */
		protected function getErrorText() {
			if($this->hasError()) {
				return '<br><font color="red">'.$this->error.'</font>';
			} else {
				return '';
			}
		}
		
		/**
		 * Clears errors for new rows in a form that is designed to have empty rows.
		 *
		 * @return VOID
		 */
		function clearErrors() {
			$this->error = null;
		}
	}
?>