<?php
	/*
	 *	Copyright 2007 John Oren
	 *	
	 *	Licensed under the Apache License, Version 2.0 (the "License");
	 *	you may not use this file except in compliance with the License.
	 *	You may obtain a copy of the License at
	 *		http://www.apache.org/licenses/LICENSE-2.0
	 *	Unless required by applicable law or agreed to in writing, software
	 *	distributed under the License is distributed on an "AS IS" BASIS,
	 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	 *	See the License for the specific language governing permissions and
	 *	limitations under the License.
	 */
	
	
	/**
	 * Handles display and processing for a Group field.
	 * 
	 * A group field is a set of fields that are combined together using a standard delimiter.
	 * Date fields are a good example of a group field (ex. month day year with a dilimter of '/')
	 * Page ranges are another example (start end with a delimiter of '-')
	 * $options["elements"] - number of individual fields to display
	 * $options["seperator"] - dilimiter for the fields
	 * $options["size"] - size of the form's text input fields
	 * OPTIONAL
	 * $options["maxlength"] - maximum length of text in any given input field
	 * $options["prefix"] - A prefix to prepend to the field data after it has been combined
	 * $options["suffix"] - A suffix to append to the field data after it has been combined
	 * 
	 * @author John Oren
	 * @version 1.0 December 30, 2007
	 */
	class Group extends FieldManager {
		/**
		 * Prepares this form field for display.
		 * 
		 * @param MIXED $defaultValue Default values to display in this form field
		 * @param BOOLEAN $enabled True if this form field is editable
		 * @return STRING HTML to display for the form field
		 */
		function show($defaultValue, $enabled=true) {
			$options = $this->getOptions();
			$ret = "";
			if(!is_array($defaultValue)) { //if the default value came from the database, the prefix and suffix need to be removed and the data split into parts
				$defaultValue = substr_replace($defaultValue, "", 0, strlen($options["prefix"]));
				$defaultValue = substr_replace($defaultValue, "", strlen($defaultValue)-strlen($options["suffix"]));
				$defaultValue = explode($options["seperator"], $defaultValue);
			}
			for($i=1; $i <= $options["elements"]; $i++) {
				$ret .= '<input type="text" name="'.$this->getName().Form::DELIMITER.$i.'" value="'.FieldManager::formPrep($defaultValue[$i-1]).'" size="'.$options["size"].'"';
				if(isset($options["maxlength"])) {
					$ret .= ' maxlength="'.$options["maxlength"].'"';
				}
				$ret .= '>';
				if($i != $options["elements"]) {
					$ret .= $options["seperator"];
				}
			}
			
			$ret .= $this->getErrorText();
			return $ret;
		}
		
		/**
		 * Processes this field and returns formated data ready for insertion into a SQL database.
		 * 
		 * @param ARRAY $value Values to process
		 * @param INTEGER $mode The type of this form (one of the form type constants defined in the Form class)
		 * @return STRING Formatted string ready for insertion into a SQL database
		 */
		function process($value, $mode) {
			if(!is_array($value)) {
				$this->error = "Error: The value for this group field was not an array.";
				return false;
			}
			
			$options = $this->getOptions();
			$ret = $options["prefix"];
			foreach($value as $val) {
				if(empty($val)) {
					return null;
				}
				$ret .= $val.$options["seperator"];
			}
			$ret = substr($ret, 0, strlen($ret)-strlen($options["seperator"]));
			$ret .= $options["suffix"];
			
			return FieldManager::SQLPrep($ret);
		}
	}
?>