<?php
	/*
	 *	Copyright 2007 John Oren
	 *	
	 *	Licensed under the Apache License, Version 2.0 (the "License");
	 *	you may not use this file except in compliance with the License.
	 *	You may obtain a copy of the License at
	 *		http://www.apache.org/licenses/LICENSE-2.0
	 *	Unless required by applicable law or agreed to in writing, software
	 *	distributed under the License is distributed on an "AS IS" BASIS,
	 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	 *	See the License for the specific language governing permissions and
	 *	limitations under the License.
	 */
	
	
	/**
	 * Handles display and processing for an Image field.
	 * 
	 * Allows an image to be uploaded, verifying the file extention, checking file size limitations,
	 * and ensuring that each database entry references a unique file (to facilitate cleanup when
	 * deleting entries without the overhead of a reference counter).
	 * $options["path"] - Full path (relative or absolute) to the directory to save uploaded images to.
	 * $options["ext"] - Comma delimited list of valid file extensions (3 or 4 letters maximum).
	 * $options["width"] - Width of the image shown as a preview of the uploaded image.
	 * $options["height"]- Height of the image shown as a preview of the uploaded image.
	 * 
	 * @author John Oren
	 * @version 1.0 December 30, 2007
	 */
	class Image extends FieldManager {
		/**
		 * Prepares this form field for display.
		 * 
		 * @param MIXED $defaultValue Default value to display in this form field
		 *				If this value is an image to upload, it will have the following format:
		 *				$defaultValue['name'] - The original name of the file on the client machine.
		 *				$defaultValue['type'] - The mime type of the file, if the browser provided this information. An example would be "image/gif". This mime type is however not checked on the PHP side and therefore don't take its value for granted.
		 *				$defaultValue['size'] - The size, in bytes, of the uploaded file.
		 *				$defaultValue['tmp_name'] - The temporary filename of the file in which the uploaded file was stored on the server.
		 *				$defaultValue['error'] - The error code associated with this file upload. See http://us2.php.net/manual/en/features.file-upload.errors.php for a list of error constants.
		 * @param BOOLEAN $enabled True if this form field is editable
		 * @return STRING HTML to display for the form field
		 */
		function show($defaultValue, $enabled=true) {
			//array("path"=>$path."images/Headers/", "ext"=>".bmp,.gif,.jpg,.jpeg", "width"=>225, "height"=>25)
			$options = $this->getOptions();
			if(is_array($defaultValue) || !empty($defaultValue)) {
				$val = $defaultValue;
				if(is_array($defaultValue) && isset($defaultValue['name'])) {
					$val = $defaultValue['name'];
				}
				if(is_array($defaultValue) && isset($defaultValue[0])) {
					$val = $defaultValue[0];
				}
				
				$types = explode(",", $options["ext"]);
				if($this->isCorrectType($val, $types)) {
					$ret = '<input type="hidden" name="'.$this->getName().'" value="'.$val.'">';
					$ret .= '<img src="'.$options["path"].$val.'" width="'.$options["width"].'" height="'.$options["height"].'"><br>';
				}
			}
			$ret .= '<input type="file" name="'.$this->getName().'"';
			if(!$enabled) {
				$ret .= ' readonly';
			}
			$ret .= '>';
			
			$ret .= $this->getErrorText();
			
			return $ret;
		}
		
		/**
		 * Prepares this form field for display in a list.
		 * 
		 * @param MIXED $defaultValue Default value or values to display for this field in a list
		 * @return STRING HTML to display in a list view
		 */
		function showListView($defaultValue) {
			$options = $this->getOptions();
			$ret = '<img src="'.$options["path"].$defaultValue.'" width="'.$options["width"].'" height="'.$options["height"].'">';
			
			return $ret;
		}
		
		/**
		 * Processes this field and returns formated data ready for insertion into a SQL database.
		 * 
		 * @param ARRAY $value Value to process. The format follows the $defaultValue specifications in the show method above.
		 * @param INTEGER $mode The type of this form (one of the form type constants defined in the Form class)
		 * @return STRING Formatted string ready for insertion into a SQL database
		 */
		function process($value, $mode) {
			$options = $this->getOptions();
			$path = $options["path"];
			$exts = explode(",", $options["ext"]);
			//500kb
			$maxSize = 102400*5;
			
			if(!is_array($value)) {
				$msg = "The data for the field ".$this->getName()." was not an array and could not be processed";
				ErrorLogManager::log($msg, ErrorLogManager::FATAL);
				$this->error = "A fatal error occured - Contact your System Administrator<br>This error has been logged";
				return false;
			}
			
			if(isset($value[0])) {
				if($mode == Form::DELETE) {
					$this->deleteFile($path.$value[0]);
				}
				return $value[0];
			}
			
			if(empty($value['tmp_name'])) {
				return false;
			}
			
			$value['name'] = strtolower($value['name']);
			if(!is_uploaded_file($value['tmp_name'])) {
				$msg = "The file for the field ".$this->getName()." could not be uploaded";
				ErrorLogManager::log($msg, ErrorLogManager::FATAL);
				$this->error = "A fatal error occured - Contact your System Administrator<br>This error has been logged";
				return false;
			}
			
			if($value['size'] > $maxSize) {
				$this->error = "The file was to large: Maximum size = 500kb";
				return false;
			}
			
			if(!$this->isCorrectType($value['name'], $exts)) {
				$this->error = "The file must end in one of the following extensions: ".$options["ext"];
				return false;
			}
			//if this file already exists, don't add it
			//TODO - rename and try again
			if(file_exists($path.$value['name'])) {
				$this->error = "A file with this name already exists. Remove the old file or rename the new one and try again.";
				return false;
			}
			//move the file from its temporary location into the proper directory
			if(!move_uploaded_file($value['tmp_name'], $path.$value['name'])) {
				$msg = "Could not copy the image file for field ".$this->getName()." from its temporary directory";
				ErrorLogManager::log($msg, ErrorLogManager::FATAL);
				$this->error = "A fatal error occured - Contact your System Administrator<br>This error has been logged";
				return false;
			}
			
			if($mode == Form::EDIT) {
				$this->deleteFile($path.$this->getDefaultValue());
			}
			return $value['name'];
		}
		
		/**
		 * Checks to ensure that the given file name has a file extension in the provided list of valid extensions
		 * 
		 * @param STRING $name Name of the file to check.
		 * @param ARRAY $exts Array of valid extensions (3 or 4 letters).
		 * @return BOOLEAN False if the given filename has an invalid file extension.
		 */
		protected function isCorrectType($name, $exts) {
			$ext1 = substr($name, strlen($name)-3);
			$ext2 = substr($name, strlen($name)-4);
			return in_array($ext1, $exts) || in_array($ext2, $exts);
		}
		
		/**
		 * Attempts to delete the given file.
		 * 
		 * @param STRING $file The full name and path (relative or absolute) of the file to delete.
		 * @return VOID
		 */
		private function deleteFile($file) {
			if(file_exists($file)) {
				if(!unlink($file)) {
					ErrorLogManager::log("Unable to remove the file ".$file." with form field ".$this->getName(), ErrorLogManager::FATAL);
					$this->error = "Failed to remove the file - Contact your System Administrator<br>This error has been logged";
				}
			}
		}
	}
?>