<?php
	/*
	 *	Copyright 2007 John Oren
	 *	
	 *	Licensed under the Apache License, Version 2.0 (the "License");
	 *	you may not use this file except in compliance with the License.
	 *	You may obtain a copy of the License at
	 *		http://www.apache.org/licenses/LICENSE-2.0
	 *	Unless required by applicable law or agreed to in writing, software
	 *	distributed under the License is distributed on an "AS IS" BASIS,
	 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	 *	See the License for the specific language governing permissions and
	 *	limitations under the License.
	 */
	
	
	/**
	 * Handles display and processing of a Password field.
	 * 
	 * There are no options for this field.
	 * 
	 * @author John Oren
	 * @version 1.0 December 30, 2007
	 */
	class Password extends FieldManager {
		/**
		 * Constructs a password field with a two-line title for the password confirmation box.
		 * 
		 * @param STRING $name The name of the database field this form field corresponds to.
		 * @param STRING $title The title to display for this form field.
		 * @param NULL $options There are no options for this field.
		 * @param BOOLEAN $inList True if this field should be used in a list view.
		 * @param BOOLEAN $required True if this form field is required.
		 */
		function __construct($name, $title, $options, $inList, $required=false) {
			parent::__construct($name, $title, $options, $inList, $required);
			$this->title .= "<br>Confirm:";
		}
		
		/**
		 * Prepares a form field for display.
		 * 
		 * @param STRING $defaultValue Default (hashed) value from the database. Never used.
		 * @param BOOLEAN $enabled True if this form field is editable
		 * @return STRING HTML to display for the form field
		 */
		function show($defaultValue, $enabled=true) {
			$ret = '<input type="password" name="'.$this->getName().'"';
			if(!$enabled) {
				$ret .= ' readonly';
			}
			$ret .= '><br><input type="password" name="'.$this->getName().Form::DELIMITER.'2"';
			if(!$enabled) {
				$ret .= ' readonly';
			}
			$ret .= '>';
			
			$ret .= $this->getErrorText();
			return $ret;
		}
		
		/**
		 * Prepares this form field for display in a list.
		 * 
		 * @param STRING $defaultValue Default value to display for this field in a list
		 * @return STRING Generic, default, obscured password marker text
		 */
		function showListView($defaultValue) {
			return "********";
		}
		
		/**
		 * Processes this field and returns formated data ready for insertion into a SQL database.
		 * 
		 * @param STRING $value Value to process
		 * @param INTEGER $mode The type of this form (one of the form type constants defined in the Form class)
		 * @return STRING Formatted and hashed string ready for insertion into a SQL database
		 */
		function process($value, $mode) {
			$pass1 = null;
			if(!is_array($value) || count($value) < 2) {
				$this->error = "Internal error processing the password field for ".$this->getName()." - value was not an array or value contained less than 2 elements";
				ErrorLogManager::log($this->error, ErrorLogManager::FATAL);
				$this->error = "A fatal error occured - Contact your System Administrator<br>This error has been logged";
				//in case this ever accidentally makes it into the database, store something difficult to hack.
				return FieldManager::SQLPrep(SecurityManager::encrypt(mt_rand(1,1000000000000).mt_rand(1,1000000000000).mt_rand(1,1000000000000)));
			}
			foreach($value as $val) {
				if($pass1 === null) {
					$pass1 = $val;
				} else {
					if($this->isRequired() && empty($pass1) && empty($val)) {
						if($mode != Form::ADD) {
							return $this->getDefaultValue();
						}
						$this->error = "No password was provided - you must provide a password";
					} elseif($pass1 != $val || empty($pass1)) {
						$this->error = "The passwords did not match";
					}
					$ret = SecurityManager::encrypt($pass1);
				}
			}
			
			return FieldManager::SQLPrep($ret);
		}
	}
?>