<?php
	/*
	 *	Copyright 2007 John Oren
	 *	
	 *	Licensed under the Apache License, Version 2.0 (the "License");
	 *	you may not use this file except in compliance with the License.
	 *	You may obtain a copy of the License at
	 *		http://www.apache.org/licenses/LICENSE-2.0
	 *	Unless required by applicable law or agreed to in writing, software
	 *	distributed under the License is distributed on an "AS IS" BASIS,
	 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	 *	See the License for the specific language governing permissions and
	 *	limitations under the License.
	 */
	
	
	require_once($path."OpenSiteAdmin/scripts/classes/Form.php");
	
	/**
	 * Manages processing and display for forms on a page.
	 * 
	 * @author John Oren
	 * @version 1.0 December 30, 2007
	 */
	class FormManager {
		/** @var DatabaseManager object. */
		private $SQL;
		/** @var The type of this form (one of the constants from the Form class). */
		private $formType;
		/** @var Array of forms on a single page. */
		private $forms;
		
		/** @var URL To redirect to (relative or absolute). */
		private $redir;
		/** @var Boolean - if true, the user will be redirected to $redir. */
		private $success;
		
		/**
		 * Constructs a new form manager, which manages all the forms on a page.
		 * 
		 * @param OBJECT $SQL DatabaseManager object.
		 * @param INTEGER $type One of the mode constants defined in this class.
		 * @param STRING $redir URL (relative or absolute) to send the user to on successful form processing.
		 */
		function __construct($SQL, $type, $redir=null) {
			if(!($SQL instanceof DatabaseManager)) {
				$error = "ERROR: Attempted to process a form with a database connection $SQL that does not inherit DatabaseManager";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			
			//$type = add\edit\delete - see constants
			$this->SQL = $SQL;
			$this->formType = $type;
			$this->forms = array();
			if($redir === null) {
				global $path;
				$redir = $path."admin/index.php";
			}
			$this->redir = $redir;
		}
		
		/**
		 * Adds a form to this form manager.
		 * 
		 * @param OBJECT $form The object of type Form to add.
		 * @return VOID
		 */
		function addForm($form) {
			if(!($form instanceof Form)) {
				$error = "ERROR: Attempted to add a form $form to a FormManager that does not inherit Form";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			
			$this->forms[] = $form;
		}
		
		/**
		 * Pre-processes form submitted data and files and initiates processing
		 * for all the forms in this object.
		 * 
		 * @param ARRAY $values Array of data from the $_POST globabl variable.
		 * @param ARRAY $files Array of files from the $_FILES global variable.
		 * @return VOID
		 */
		function processContent($values, $files) {
			if(!is_array($values)) {
				$values = array();
			}
			if(!is_array($files)) {
				$files = array();
			}
			if(!(isset($values["submit"]) || isset($values["update"])) || count($this->forms) == 0) {
				return;
			}
			
			//preprocess the data by row ID
			$data = $this->getData($values, $files);
			
			$this->success = true;
			foreach($this->forms as $form) {
				$this->success = $form->processForm($this->SQL, $this->formType, $data) && $this->success;
			}
		}
		
		/**
		 * Pre-processes form submitted data and files and initiates display
		 * for all the forms in this object.
		 * 
		 * @param ARRAY $values Array of data from the $_POST globabl variable.
		 * @param ARRAY $files Array of files from the $_FILES global variable.
		 * @return VOID Redirects the user if no errors were encountered.
		 */
		function showForm($values, $files) {
			if(!is_array($values)) {
				$values = array();
			}
			if(!is_array($files)) {
				$files = array();
			}
			
			//preprocess the data by row ID
			$data = $this->getData($values, $files);
			
			if(count($this->forms) > 0) {
				print '<form method="post" enctype="multipart/form-data" action="">';
				foreach($this->forms as $form) {
					$form->displayForm($this->formType, $data);
				}
				print '<br>';
				print '<input type="submit" name="submit" value="'.$this->getSubmitText($this->formType).'"';
				if($this->formType == Form::DELETE) {
					print ' onClick="return(window.confirm(\'Are you sure you want to permanently delete this?\'))"';
				}
				print '>';
				print '</form>';
			}
			
			if($this->success) {
				header("Location:".$this->redir."?text=The%20form%20was%20submitted%20succesfully!");
			}
		}
		
		/**
		 * Pre-processes form submitted data and files into an array format
		 * expected by Form process or display methods.
		 * 
		 * @param ARRAY $values Array of data from the $_POST globabl variable.
		 * @param ARRAY $files Array of files from the $_FILES global variable.
		 * @return ARRAY of formated data.
		 */
		function getData($values, $files) {
			$data = array();
			$globals = array();
			foreach($values as $key=>$val) {
				$key = explode(Form::DELIMITER, $key);
				if(count($key) < 2) {
					$globals[$key[0]] = $val;
				}
				$data[$key[0]][$key[1]][] = $val;
			}
			foreach($files as $key=>$val) {
				$key = explode(Form::DELIMITER, $key);
				$data[$key[0]][$key[1]][] = $val;
			}
			foreach($data as $key=>$val) {
				$data[$key] += $globals;
			}
			return $data;
		}
		
		/**
		 * Returns the text for a submit button for a given form type.
		 * 
		 * @param INTEGER $mode One of the mode constants defined in this class.
		 * @return STRING Submit button text.
		 */
		function getSubmitText($mode) {
			switch($mode) {
				case Form::ADD:
					return "Add Entry";
				case Form::EDIT:
					return "Edit Entry";
				case Form::DELETE:
					return "Delete Entry";
				default:
					return "Unknown Mode "+$mode;
			}
		}
	}
?>