<?php
	/*
	 *	Copyright 2007 John Oren
	 *	
	 *	Licensed under the Apache License, Version 2.0 (the "License");
	 *	you may not use this file except in compliance with the License.
	 *	You may obtain a copy of the License at
	 *		http://www.apache.org/licenses/LICENSE-2.0
	 *	Unless required by applicable law or agreed to in writing, software
	 *	distributed under the License is distributed on an "AS IS" BASIS,
	 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	 *	See the License for the specific language governing permissions and
	 *	limitations under the License.
	 */
	
	
	/**
	 * Handles display and processing for a list-style form.
	 * 
	 * A list form displays multiple copies of a single row in a list-style format.
	 * Titles for each field are shown above all the rows, with each row following on its own line.
	 * This form makes use of the form's optional parameter by adding the specified number of extra
	 * empty rows to the end of the list. If this field is left blank or 0, no extra rows are added.
	 * 
	 * @author John Oren
	 * @version 1.0 December 30, 2007
	 */
	class Form_List extends Form {
		/**
		 * Displays this form's data row by row in an HTML form. If their are any
		 * extra blank rows, an update button is added at the end of the form to add
		 * additional blank rows.
		 * 
		 * @param INTEGER $mode One of the mode constants defined in this class.
		 * @param ARRAY $data Array of form data in the following format:
		 *				$data[rowID][fieldName] = value
		 * @return VOID
		 */
		function displayForm($mode, $data) {
			print '<table>';
			$rowFields = $this->rows[0]->getFields();
			print '<tr>';
			foreach($rowFields as $field) {
				print '<td>'.$field->getTitle().'</td>';
			}
			print '</tr>';
			
			foreach($this->rows as $row) {
				print '<tr>';
				foreach($row->getFields() as $field) {
					$fieldName = $field->getName();
					$value = $data[$row->getRowID()][$field->getFieldName()];
					
					if(count($value) == 1) {
						$value = $value[0];
					}
					if(count($value) == 0) {
						$value = null;
					}
					
					if($row->isEmpty()) {
						$field->clearErrors();
					}
					print '<td>'.$field->show($value, $mode != Form::DELETE).'</td>';
				}
				print '</tr>';
			}
			
			print '</table>';
			if(is_int($this->getOptions()) && $this->getOptions() > 0) {
				print '<br>';
				print '<input type="submit" name="update" value="Add more rows">';
				print '<br>';
			}
		}
		
		/**
		 * Processes all of this form's rows with the given data.
		 * 
		 * @param OBJECT $SQL DatabaseManager object.
		 * @param INTEGER $mode One of the mode constants defined in this class.
		 * @param ARRAY $data Array of form data in the following format:
		 *				$data[rowID][fieldName] = value
		 * @return BOOLEAN False if this form had errors.
		 */
		function processForm($SQL, $mode, $data) {
			if(!($SQL instanceof DatabaseManager)) {
				$error = "ERROR: Attempted to process a Form_List with a database connection $SQL that does not inherit DatabaseManager";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			
			switch($mode) {
				case Form::ADD:
				case Form::EDIT:
					$type = "update";
					break;
				case Form::DELETE:
					$this->options = 2;
					$type = "delete";
					break;
				default:
					$msg = "FATAL ERROR: Unsupported form type ".$mode;
					print $msg;
					ErrorLogManager::log($msg, ErrorLogManager::FATAL, $SQL);
					die("This error has been logged. Please contact your system administrator.");
					break;
			}
			
			$option = $this->getOptions();
			$success = true;
			$emptyRows = 1;
			$row = $this->rows[0];
			//add in rows until there are [option] number of empty rows in the form
			while($emptyRows < $option) {
				$try = $row->$type($SQL, $data[$row->getRowID()], $mode);
				if($row->isEmpty()) {
					$emptyRows++;
				} elseif(!$try) {
					$success = false;
				}
				$row = clone $row;
				$this->addRow($row);
			}
			
			return $success;
		}
	}
?>