<?php
	/*
	 *	Copyright 2007 John Oren
	 *	
	 *	Licensed under the Apache License, Version 2.0 (the "License");
	 *	you may not use this file except in compliance with the License.
	 *	You may obtain a copy of the License at
	 *		http://www.apache.org/licenses/LICENSE-2.0
	 *	Unless required by applicable law or agreed to in writing, software
	 *	distributed under the License is distributed on an "AS IS" BASIS,
	 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	 *	See the License for the specific language governing permissions and
	 *	limitations under the License.
	 */
	
	
	/**
	 * Manages processing and display for lists based on a row's database information.
	 * 
	 * @author John Oren
	 * @version 1.0 December 30, 2007
	 */
	class ListManager {
		/** @var DatabaseManager object. */
		private $SQL;
		
		/** @var Array of filters to apply to the database list. */
		private $filters;
		/** @var name of the database column to use in a SQL ORDER BY clause to apply to the database list. */
		private $order;
		
		/**
		 * Constructs a new list (does not initiate any display or processing).
		 * 
		 * @param OBJECT $SQL DatabaseManager object.
		 */
		function __construct($SQL) {
			if(!($SQL instanceof DatabaseManager)) {
				$error = "ERROR: Attempted to construct a ListManager with a database connection $SQL that does not inherit DatabaseManager";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			
			$this->SQL = $SQL;
			$this->filters = array();
			$this->order = null;
		}
		
		/**
		 * Adds a filter to the list being displayed.
		 * 
		 * @param OBJECT $filter The object of type Filter to add.
		 * @return VOID
		 */
		function addFilter($filter) {
			if(!($filter instanceof Filter)) {
				$error = "ERROR: Attempted to add a filter $filter to a list that does not inherit Filter";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			
			$this->filters[] = $filter;
		}
		
		/**
		 * Sets the MySQL ORDER BY clause (The phrase "ORDER BY" is automatically included).
		 * 
		 * @param STRING $order Name of the column to order the display by.
		 * @return VOID
		 */
		function setOrderBy($order) {
			$this->order = $order;
		}
		
		/**
		 * Generates and returns a list (HTML table) of data using the given row as a template.
		 * 
		 * @param OBJECT $row RowManager object to use as a template for fields and field titles.
		 * @param STRING $linkField The database field to use as a link to allow for editing\deleting a particular field.
		 * @param STRING $QS Any text to apply to the end of links. An '&' is prepended to this if it is not empty.
		 * @param ARRAY $data Form data from filter forms.
		 * @return STRING HTML Table containing the list data.
		 */
		function generateList($row, $linkField, $QS, $data=null) {
			if(!($row instanceof RowManager)) {
				$error = "ERROR: Attempted to generate a form from a row $row that does not inherit RowManager";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			
			if(!empty($QS)) {
				$QS = "&".$QS;
			}
			
			$fields = $row->getFields();
			$ret = $this->generateFilterForms($this->filters, $data);
			
			$where = RowManager::getFilterClause($this->filters);
			if(!empty($this->order)) {
				$order = "ORDER BY `".$this->order."`";
			}
			
			$sql = "select * from `".$row->getTableName()."` ".$where." ".$order;
			$result = mysql_query($sql, $this->SQL->getLinkResource());
			print mysql_error()."<br>";
			
			$ret .= '<table border="0" cellspacing="0" cellpadding="1">';
			//add list headers
			$ret .= '<tr>';
			foreach($fields as $field) {
				if($field->isInList()) {
					$ret .= '<td>'.$field->getTitle().'</td>';
				}
			}
			$ret .= '</tr>';
			
			//add list data
			while($entry = mysql_fetch_assoc($result)) {
				$ret .= "<tr>";
				foreach($fields as $field) {
					if(!$field->isInList()) {
						continue;
					}
					$ret .= "<td>";
					$fieldName = $field->getFieldName();
					$val = $field->showListView($entry[$fieldName]);
					if($fieldName == $linkField) {
						$ret .= '<a href="?id='.$entry[$row->getPrimaryKeyField()].$QS.'">'.$val.'</a>';
					} else {
						$ret .= $val;
					}
					$ret .= "</td>";
				}
				$ret .= "</tr>";
			}
			
			$ret .= '</table>';
			return $ret;
		}
		
		/**
		 * Generates forms from any filters that may be applied to this form.
		 * 
		 * @param ARRAY $filters Array of filters to generate forms for.
		 * @param ARRAY $data Form data to use to populate filter data.
		 * @return STRING HTML form to use for filtering the list.
		 */
		protected function generateFilterForms($filters, $data) {
			if(count($filters) == 0) {
				return "";
			}
			$ret = '<form method="post" action="">';
			
			foreach($filters as $filter) {
				$filter->setDefaultValue(array_shift($data));
				$ret .= $filter->getForm($this->SQL)."<br>";
			}
			
			$ret .= '<input type="submit" name="filter" value="Apply Filter';
			if(count($this->filters) > 1) {
				$ret .= 's';
			}
			$ret .= '">';
			$ret .= '</form>';
			
			return $ret;
		}
	}
?>