<?php
	/*
	 *	Copyright 2007 John Oren
	 *	
	 *	Licensed under the Apache License, Version 2.0 (the "License");
	 *	you may not use this file except in compliance with the License.
	 *	You may obtain a copy of the License at
	 *		http://www.apache.org/licenses/LICENSE-2.0
	 *	Unless required by applicable law or agreed to in writing, software
	 *	distributed under the License is distributed on an "AS IS" BASIS,
	 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	 *	See the License for the specific language governing permissions and
	 *	limitations under the License.
	 */
	
	
	/**
	 * Provides permission checking and encryption methods for the site.
	 * 
	 * @author John Oren
	 * @version 1.0 December 30, 2007
	 */
	class SecurityManager {
		/** @var DatabaseManager object. */
		private $SQL;
		/** @var Array of permissions levels created by the setPermissions() method. */
		private $permissions;
		/** @var Array of error messages created by the setPermissions() method. */
		private $errorMessages;
		
		/**
		 * Sets up the security manager with access to a database and an optional page to check permissions on.
		 * 
		 * @param OBJECT $sql DatabaseManager object.
		 * @param STRING $page The name of a page to check.
		 * @param STRING $mode An optional suffix to add to the page name, such as Add, Edit, or Delete.
		 */
		function __construct($sql, $page="", $mode="") {
			if(!($sql instanceof DatabaseManager)) {
				$error = "ERROR: Attempted to construct a security manager with a database connection $sql that does not inherit DatabaseManager";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			
			$this->SQL = $sql;
			$this->setPermissions();
			if(!empty($page)) {
				$this->validatePage($mode.$page);
			}
		}
		
		/**
		 * Returns an array keyed by a user level's access level with values for the name of the particular access level.
		 * 
		 * @return ARRAY $ret[levelNum] = levelName
		 */
		static function getAccessLevels() {
			return array(1=>"Developer", 2=>"Administrator", 3=>"User");
		}
		
		/**
		 * Sets arrays of permission levels and error messages corresponding to the keys from getAccessLevels()
		 * for every page registered with the system.
		 * 
		 */
		function setPermissions() {
			$permissions = array();
			
			$result = mysql_query("select * from access", $this->SQL->getLinkResource());
			while($row = mysql_fetch_assoc($result)) {
				$this->permissions[$row["pageName"]] = $row["minLevel"];
				$this->errorMessages[$row["pageName"]] = $row["message"];
			}
		}
		
		/**
		 * Acts like PHP's basename($url, ".php") function, but also strips off any _GET variables in the URL.
		 * 
		 * @param STRING $url URL to process.
		 * @return STRING page name (no path information, no file extension, no _GET variable or anchor (#) information).
		 */
		function getBaseName($url) {
			$ret = basename($url);
			$ret = explode(".php", $ret);
			return $ret[0];
		}
		
		/**
		 * Encrypts the given string using the specified hashing algorithm.
		 * See the end of /OpenSiteAdmin/scripts/php.php for a list of available hashing algorithms.
		 * If the first option is not available, try the algorithms in order from top to bottom for best
		 * security. The md5 function is provided as a last resort and is not recommended for applications
		 * needing to withstand organized security threats.
		 * 
		 * @param STRING $string The string to encrypt.
		 * @return STRING Encrypted text (usually significantly longer than the input text).
		 */
		static function encrypt($string) {
			return hash("sha512", $string);
			//return hash("sha384", $string);
			//return hash("whirlpool", $string);
			//return md5($string);
		}
		
		/**
		 * Verifies that the current user is allowed to view at least one of an add, edit, or delete page.
		 * 
		 * @param STRING $pageSuffix Suffix for add, edit, and delete (ex. add$pageSuffix).
		 * @return BOOLEAN False if the current user is not allowed to view the specified 'row' of pages.
		 */
		function isRowVisible($pageSuffix) {
			if(!isset($_SESSION["permissions"]))
				return false;
			
			$lvl = $_SESSION["permissions"];
			
			$page = "add".$pageSuffix;
			if($lvl <= $this->permissions[$page])
				return true;
			$page = "edit".$pageSuffix;
			if($lvl <= $this->permissions[$page])
				return true;
			$page = "delete".$pageSuffix;
			if($lvl <= $this->permissions[$page])
				return true;
			
			return false;
		}
		
		/**
		 * Verifies that the current user is allowed to view the specified page.
		 * 
		 * @param STRING $page The page to check permissions for.
		 * @return BOOLEAN False if the user is not allowed to view the page or
		 *				   exits with an error message if the page has no permissions.
		 */
		function isPageVisible($page) {
			if(!isset($_SESSION["username"])) {
				return false;
			}
			
			if(!isset($this->permissions[$page])) {
				$page = $this->getBaseName($_SERVER["REQUEST_URI"]);
				if(!isset($this->permissions[$page])) {
					die("This page has no authorization level. No one can view it. Please contact a developer to add ".$page." into the authorization system");
				}
			}
			if(isset($_SESSION["permissions"]) && $_SESSION["permissions"] <= $this->permissions[$page]) {
				return true;
			} else {
				return false;
			}
		}
		
		/**
		 * Checks if a page is visible. If it is not visible, redirects the user to the site's login page.
		 * 
		 * @param STRING $page The page to check permissions for.
		 * @return VOID Redirects the user if they cannot access the specified page.
		 */
		function validatePage($page) {
			if(!isset($_SESSION["username"])) {
				global $path;
				die(header("Location:".$path."OpenSiteAdmin/login.php"));
			}
			if(!$this->isPageVisible("admin") && $page != "index") {
				global $path;
				die(header("Location:".$path."admin/index.php?text=".$this->errorMessages["admin"]));
			}
			if(!$this->isPageVisible($page)) {
				global $path;
				die(header("Location:".$path."admin/index.php?text=".$this->errorMessages[$page]));
			}
		}
	}
?>